# -*- mode: python; coding: utf-8 -*-
# Pigment rendering library
#
# Copyright © 2006 Fluendo Embedded S.L. <www.fluendo.com>
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2. See "LICENSE.GPL" in
# the root of this distribution including a special exception to use
# Pigment with Fluendo's plugins.
#
# The GPL part of Pigment is also available under a commercial licensing
# agreement from Fluendo. See "LICENSE.Pigment" in the root directory
# of this distribution package for details on that license.

"""
"""

from pgm.form import color, paint

# Coordinate indices
X, Y = 0, 1


class Gradient(paint.Paint):
    """Gradient base class.

    Not meant to be instantiated directly.
    """

    def __init__(self):
        paint.Paint.__init__(self)
        self._color_stop = []

    ### Public methods

    # FIXME
    def add_color_stop(self, offset, _color=color.WHITE):
        """
        """
        clamped_offset = max(0.0, min(1.0, offset))
        self._color_stop.append((clamped_offset, _color))

    # FIXME
    def get_color_stop(self):
        """
        """
        return self._color_stop

    # FIXME
    def set_color_stop(self, index, offest, _color):
        """
        """
        if index < len(self._color_stop):
            clamped_offset = max(0.0, min(1.0, offset))
            self._color_stop[index] = (clamped_offset, _color)

    # FIXME
    def del_color_stop(self, index):
        """
        """
        if index < len(self._color_stop):
            self._color_stop.pop(index)

    ### Private methods

    def _add_shape(self, _shape):
        if isinstance(_shape, shape.Shape):
            self._shape.append(_shape)

    def _del_shape(self, _shape):
        if _shape in self._shape:
            self._shape.remove(_shape)


# FIXME: Fix the API
class LinearGradient(Gradient):
    """Linear Gradient class
    """

    def __init__(self, a, b):
        Gradient.__init__(self)
        self._a = (max(0.0, min(1.0, a[X])), max(0.0, min(1.0, a[Y])))
        self._b = (max(0.0, min(1.0, b[X])), max(0.0, min(1.0, b[Y])))

    # FIXME
    def get_a(self):
        """
        """
        return self._a

    # FIXME
    def get_b(self):
        """
        """
        return self._b


# FIXME: Fix the API
class RadialGradient(Gradient):
    """Radial Gradient class
    """

    def __init__(self, a_center, a_radius, b_center, b_radius):
        Gradient.__init__(self)
        self._a_center = (max(0.0, min(1.0, a_center[X])),
                          max(0.0, min(1.0, a_center[Y])))
        self._a_radius = max(0.0, min(1.0, a_radius))
        self._b_center = (max(0.0, min(1.0, b_center[X])),
                          max(0.0, min(1.0, b_center[Y])))
        self._b_radius = max(0.0, min(1.0, b_radius))
