# -*- mode: python; coding: utf-8 -*-
# Pigment rendering library
#
# Copyright © 2006 Fluendo Embedded S.L. <www.fluendo.com>
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2. See "LICENSE.GPL" in
# the root of this distribution including a special exception to use
# Pigment with Fluendo's plugins.
#
# The GPL part of Pigment is also available under a commercial licensing
# agreement from Fluendo. See "LICENSE.Pigment" in the root directory
# of this distribution package for details on that license.

"""
"""

from pgm import log, render
from pgm.form import media

from PIL import Image as pil
from PIL import ImageFile

import os.path
import tempfile

# FIXME: Image internal pixel buffer is in RGBA mode

class Image(media.Media):
    """
    """

    def __init__(self, filename=None, data=None, pixels=None, size=None, format=None):
        media.Media.__init__(self, media.BILINEAR, 1.33)
        
        if filename:
            filename = os.path.normpath(filename)
            self._pil_open_from_file(filename)
            self._filename = filename
            
        elif data:
            self._pil_open_from_data(data)
            self._filename = None

        elif pixels and size and format:
            # FIXME: check consistency
            if True:
                self._filename = None
                self._pil_object = pil.frombuffer(format,
                                                  size,
                                                  pixels,
                                                  "raw",
                                                  format, 0, 1)
            else:
                raise Exception("Pixel buffer inconsistent with passed size and format")


    def __repr__(self):
        return "<image.Image(file=%r)>" % self._filename

    # Property definitions

    def pixels__get(self):
        """The pixel data"""
        return self._pil_object.tostring('raw', self._pil_object.mode, 0, 1)

    def filename__get(self):
        """The file name of the image if it exists"""
        return self._filename

    def format__get(self):
        """The pixel image format"""
        renderer = render.get_renderer()
        formats = {'RGBA': renderer.MEDIA_RGBA, }
        return formats.get(self._pil_object.mode)

    def size__get(self):
        """The size"""
        return self._pil_object.size

    # Public methods

    def resize(self, size):
        # TODO: check if thumbnail quality is still ok
        #self._pil_object.thumbnail((size, size), pil.ANTIALIAS)
        self._pil_object.thumbnail((size, size), pil.NEAREST)

    def rotate(self, angle):
        self._pil_object = self._pil_object.rotate(angle)

    def crop(self, box):
        self._pil_object = self._pil_object.crop(box)


    # Private methods

    def _pil_open_from_file(self, picture_path):
        log.dbg("Loading image at %s", picture_path)
        try:
            img = pil.open(picture_path)
        except IOError, error:
            log.info("Image._pil_open_from_file(): Error loading %r : %s",
		     picture_path, error)
            raise

        if 'interlace' in img.info and img.info['interlace']:
            raise TypeError(_("Can't load %s. Interlaced pictures are not supported" % picture_path))

        if img.mode != 'RGBA':
            img = img.convert(mode='RGBA')
        
        self._pil_object = img

    def _pil_open_from_data(self, data):
        # TODO: for now on, PIL 1.1.5 and 1.1.6, this method does not
        # work for PNG files at least
        # cf. http://www.mail-archive.com/image-sig@python.org/msg01153.html
        """
        try:
            p = ImageFile.Parser()
            p.feed(data)
            img = p.close()
        """

        # WORKAROUND bug in PIL: save the data to a temporary file
        file = tempfile.TemporaryFile(mode='w+b')
        file.write(data)
        file.seek(0)

        try:
            img = pil.open(file)
        # END OF THE WORKAROUND
        except IOError, error:
            log.info("Image._pil_open_from_memory(): Error loading image from memory : %s", \
                      error)
            raise

        if 'interlace' in img.info and img.info['interlace']:
            raise TypeError(_("Interlaced pictures are not supported"))

        if img.mode != 'RGBA':
            img = img.convert(mode='RGBA')

        self._pil_object = img
