# -*- mode: python; coding: utf-8 -*-
# Pigment rendering library
#
# Copyright © 2006 Fluendo Embedded S.L. <www.fluendo.com>
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2. See "LICENSE.GPL" in
# the root of this distribution including a special exception to use
# Pigment with Fluendo's plugins.
#
# The GPL part of Pigment is also available under a commercial licensing
# agreement from Fluendo. See "LICENSE.Pigment" in the root directory
# of this distribution package for details on that license.

"""
"""

from pgm import render, widget
from pgm.util import maths, string_tools
from pgm.layout import leaf, window
from pgm.form import color, shape

# Slant constants
SLANT_NORMAL, SLANT_ITALIC = 0, 1
# Weight constants
WEIGHT_NORMAL, WEIGHT_BOLD = 0, 1
# Size clamping constants
MIN_SIZE, MAX_SIZE = 10, 100


class Label(leaf.Leaf):
    """
    """

    def __init__(self, size=(150,50), position=(0,0), text='Pigment label',
		 font_name='Sans', font_size=12, slant=SLANT_NORMAL,
		 weight=WEIGHT_NORMAL, color=color.Color()):
        self._font = render.get_renderer().create_font()
	leaf.Leaf.__init__(self, size, position)
        self._font_name = font_name
        self._font_size = maths.clamp(font_size, MIN_SIZE, MAX_SIZE)
        self._slant = slant
        self._weight = weight
        self._color = color
	self._font.set_font(self._font_name, self._weight, self._slant,
			    self._font_size)
        self.text = text
        self._font.set_color(self._color.r, self._color.g, self._color.b,
                             self._color.a)

    # Property definitions

    def text__get(self):
        """The text label to render"""
        return self._text

    def text__set(self, text):
        text = string_tools.fs_encode(text)
        self._text = text
        self._font.set_label(text)

    def font_name__get(self):
        """The font name"""
        return self._font_name

    def font_name__set(self, font_name):
        self._font_name = font_name
        self._font.set_font(self._font_name, self._weight, self._slant,
			    self._font_size)

    def font_size__get(self):
        """The font size"""
        return self._font_size

    def font_size__set(self, font_size):
	self._font_size = maths.clamp(font_size, MIN_SIZE, MAX_SIZE)
        self._font.set_font(self._font_name, self._weight, self._slant,
			    self._font_size)

    def weight__get(self):
        """The font weight"""
        return self._weight

    def weight__set(self, weight):
        self._weight = weight
        self._font.set_font(self._font_name, self._weight, self._slant,
			    self._font_size)

    def slant__get(self):
        """The font slant"""
        return self._slant

    def slant__set(self, slant):
        self._slant = slant
        self._font.set_font(self._font_name, self._weight, self._slant,
			    self._font_size)

    def color__get(self):
        """The color"""
        return self._color

    def color__set(self, color):
        self._color = color
        self.opacity = color.a
        self._font.set_color(self._color.r, self._color.g, self._color.b,
                             self._color.a)

    def text_size__get(self):
        """The real text size"""
        return self._font.get_size()

    # Property overriding

    def size__set(self, size):
        widget.Widget.size__set(self, size)
        self._font.set_size(*size)

    def position__set(self, position):
        widget.Widget.position__set(self, position)
        self._font.set_position(*position)

    def ordering__set(self, ordering):
        widget.Widget.ordering__set(self, ordering)
        if self._window:
            self._window._rdr_window.del_drawable(self._font)
            self._window._rdr_window.add_drawable(self._font,
                                                  widget.order[self._ordering])

    def x__set(self, x):
        widget.Widget.x__set(self, x)
        self._font.set_position(*self.position)

    def y__set(self, y):
        widget.Widget.y__set(self, y)
        self._font.set_position(*self.position)
        
    def tx__set(self, tx):
        widget.Widget.tx__set(self, tx)
        self._font.set_translation(*self.translation)

    def ty__set(self, ty):
        widget.Widget.ty__set(self, ty)
        self._font.set_translation(*self.translation)

    def tz__set(self, tz):
        widget.Widget.tz__set(self, tz)
        self._font.set_translation(*self.translation)

    def translation__set(self, translation):
        widget.Widget.translation__set(self, translation)
        self._font.set_translation(*self.translation)

    def rx__set(self, rx):
        widget.Widget.rx__set(self, rx)
        self._font.set_rotation(*self.rotation)

    def ry__set(self, ry):
        widget.Widget.ry__set(self, ry)
        self._font.set_rotation(*self.rotation)

    def rz__set(self, rz):
        widget.Widget.rz__set(self, rz)
        self._font.set_rotation(*self.rotation)

    def rotation__set(self, rotation):
        widget.Widget.rotation__set(self, rotation)
        self._font.set_rotation(*self.rotation)

    def opacity__set(self, opacity):
        widget.Widget.opacity__set(self, opacity)
        self._color.a = opacity
        self._font.set_color(self._color.r, self._color.g, self._color.b,
                             self._color.a)

    def opacity_offset__set(self, opacity_offset):
        widget.Widget.opacity_offset__set(self, opacity_offset)
        opacity = self.opacity__get()
        self._color.a = opacity
        self._font.set_color(self._color.r, self._color.g, self._color.b,
                             self._color.a)
        
    def x_align__set(self, x_align):
       leaf.Leaf.x_align__set(self, x_align)
       self._font.set_alignment(self._x_align, self._y_align)

    def y_align__set(self, y_align):
       leaf.Leaf.y_align__set(self, y_align)
       self._font.set_alignment(self._x_align, self._y_align)

    def align__set(self, align):
       leaf.Leaf.align__set(self, align)
       self._font.set_alignment(self._x_align, self._y_align)

    def visible__set(self, visible):
        widget.Widget.visible__set(self, visible)
        if visible:
            self._font.show()
        else:
            self._font.hide()

    # Private methods

    def _set_window(self, window):
        self._window = window
        window._rdr_window.add_drawable(self._font,
                                        widget.order[self._ordering])

    def _unset_window(self):
        if self._window:
            self._window._rdr_window.del_drawable(self._font)
        self._window = None
