# -*- mode: python; coding: utf-8 -*-
# Pigment rendering library
#
# Copyright © 2006 Fluendo Embedded S.L. <www.fluendo.com>
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2. See "LICENSE.GPL" in
# the root of this distribution including a special exception to use
# Pigment with Fluendo's plugins.
#
# The GPL part of Pigment is also available under a commercial licensing
# agreement from Fluendo. See "LICENSE.Pigment" in the root directory
# of this distribution package for details on that license.

"""
"""

from pgm.util import maths
from pgm.form import component

# Interpolation constants
NEAREST, BILINEAR = 0, 1


class Media(component.Component):
    """
    """

    def __init__(self, interpolation, aspect_ratio):
        component.Component.__init__(self)
        self.interpolation = interpolation
        self.aspect_ratio = aspect_ratio

    # Property definitions

    def interpolation__get(self):
        """The interpolation method used to resize the media"""
        return self._interpolation

    def interpolation__set(self, interpolation):
        if interpolation == NEAREST or interpolation == BILINEAR:
            self._interpolation = interpolation
        else:
            interpolation = BILINEAR

    def aspect_ratio__get(self):
        """The pixel aspect ratio of the media"""
        return self._aspect_ratio

    def aspect_ratio__set(self, aspect_ratio):
        self._aspect_ratio = maths.clamp(aspect_ratio, 0.0, 10.0)
