# -*- mode: python; coding: utf-8 -*-
# Pigment rendering library
#
# Copyright © 2006 Fluendo Embedded S.L. <www.fluendo.com>
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2. See "LICENSE.GPL" in
# the root of this distribution including a special exception to use
# Pigment with Fluendo's plugins.
#
# The GPL part of Pigment is also available under a commercial licensing
# agreement from Fluendo. See "LICENSE.Pigment" in the root directory
# of this distribution package for details on that license.

import sys, os

DEBUG=2
INFO=1
QUIET=0

LEVEL=0

def set_level(level):
    """
    level:

    - 0 => no info or dbg output (quiet mode)
    - 1 => info output
    - 2 => info + dbg output
    
    """
    global LEVEL
    LEVEL = level

def get_level():
    global LEVEL
    return LEVEL

def dbg(format, *args):
    level = get_level()
    if level > 1:
        fr = sys._getframe(1)
        filename = os.path.basename(fr.f_code.co_filename)
        func = fr.f_code.co_name
        lineno = fr.f_lineno
        callable_name = '%s:%s():%d' % (filename, func, lineno)
        sys.stderr.write(('%-40s: ' + format + '\n')
                         % ((callable_name,) + args))

def info(format, *args):
    level = get_level()
    if level > 0:
        print format % args

def warn(format, *args):
    level = get_level()
    if level > 0:
        sys.stderr.write(('Warning: ' + format + '\n') % args)

def err(format, *args):
    sys.stderr.write(('Error: ' + format + '\n') % args)
    sys.exit(1)
