# -*- mode: python; coding: utf-8 -*-
# Pigment rendering library
#
# Copyright © 2006 Fluendo Embedded S.L. <www.fluendo.com>
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2. See "LICENSE.GPL" in
# the root of this distribution including a special exception to use
# Pigment with Fluendo's plugins.
#
# The GPL part of Pigment is also available under a commercial licensing
# agreement from Fluendo. See "LICENSE.Pigment" in the root directory
# of this distribution package for details on that license.

from pgm import log


global registry
registry = {}

global REGISTRY_ENABLED
REGISTRY_ENABLED=False

def enable_registry():
    global REGISTRY_ENABLED
    REGISTRY_ENABLED = True

def disable_registry():
    global REGISTRY_ENABLED
    REGISTRY_ENABLED = False
    

class WrongArgument(Exception):

    def __init__(self, arg, spec, position):
        Exception.__init__(self)
        self.arg = arg
        self.spec = spec
        self.position = position
        
    def __str__(self):
        msg = "Expected an %r at position %s, got %r (%r) instead."
        msg = msg % (self.spec.__name__, self.position, self.arg, type(self.arg))
        return msg

class Signal(object):
    """
    """
    
    def __init__(self, name, arg_spec=None, registry_override=None):
        self._name = name
        self._receivers = []
        self._default_func = []
        self.un_neutralized = []
        if not arg_spec:
            arg_spec = ()
        self._arg_spec = arg_spec

        global registry, REGISTRY_ENABLED
        if registry_override is not None:
            registry_enabled = registry_override
        else:
            registry_enabled = REGISTRY_ENABLED
        
        if registry_enabled:
            if name not in registry:
                registry[name] = self
            else:
                s = registry[name]
                self._receivers = s._receivers
                self._default_func = s._default_func
                self._arg_spec = s._arg_spec
            
    def __repr__(self):
        r = '<Signal %r %d receivers>' % (self._name, len(self.get_receivers()))
        return r

    def check_args(self, args):
        i = 0
        for arg in args:
            spec = self._arg_spec[i]
            if not isinstance(arg, spec):
                raise WrongArgument(arg, spec, i)
            i += 1
            
    def emit(self, *data, **kw):
        for (receiver, init_data, type_check) in self._receivers:
            if self.is_receiver_neutralized(receiver):
                continue
            if type_check:
                self.check_args(data)
            if init_data is not None:
                data = data + init_data
            log.dbg("%r is calling %r" % (self, receiver))
            receiver(*data, **kw)

    def connect(self, func, data=None, type_check=True):
        if callable(func):
            if data is not None and type(data) not in (list, tuple):
                data = (data,)
            receiver = (func, data, type_check)
            if receiver not in self._receivers:
                self._receivers.append(receiver)

    def disconnect(self, func):
        for (receiver, data, type_check) in self._receivers:
            if receiver == func:
                self._receivers.remove((func, data, type_check))
                return True
        return False

    def get_receivers(self):
        return self._receivers

    def neutralize(self, excepted_callbacks):
        self.un_neutralized = excepted_callbacks

    def restore(self):
        self.un_neutralized = []

    def is_receiver_neutralized(self, receiver):
        n = False
        if self.un_neutralized:
            n = receiver not in self.un_neutralized
        return n
        
            
