# -*- mode: python; coding: utf-8 -*-
# Pigment rendering library
#
# Copyright © 2006 Fluendo Embedded S.L. <www.fluendo.com>
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2. See "LICENSE.GPL" in
# the root of this distribution including a special exception to use
# Pigment with Fluendo's plugins.
#
# The GPL part of Pigment is also available under a commercial licensing
# agreement from Fluendo. See "LICENSE.Pigment" in the root directory
# of this distribution package for details on that license.

"""
Pigment utility module 
"""

import math

PI, PI2 = math.pi, math.pi/2.0


def clamp(x, low, high):
    """Clamp a value in the range [low, high].
    """
    return max(low, min(high, x))


def lerp(a, b, factor):
    """Linear interpolation applied on a number.
    """
    return clamp(factor, 0.0, 1.0) * (b - a) + a


def lerp_seq(a, b, factor):
    """Linear interpolation applied on sequence like list or tuple.
    """
    diff = map(lambda x,y: x-y,  b, a)
    mult = map(lambda x: clamp(factor, 0.0, 1.0)*x, diff)
    return map(lambda x,y: x+y, mult, a)
