# -*- coding: utf-8 -*-
# Pigment Rendering Toolkit
#
# Copyright (C) 2006 Fluendo Embedded S.A. <www.fluendo.com>
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Pigment with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commerical licensing
# agreement from Fluendo.
# See "LICENSE.Pigment" in the root directory of this distribution package
# for details on that license.

# see http://developer.gnome.org/doc/API/2.0/glib/glib-running.html

import os
import locale


if "G_FILENAME_ENCODING" in os.environ:
    FSCODING = os.environ["G_FILENAME_ENCODING"].split(",")[0]
    if FSCODING == "@locale":
        FSCODING = locale.getpreferredencoding()
elif "G_BROKEN_FILENAMES" in os.environ:
    FSCODING = locale.getpreferredencoding()
else:
    FSCODING = "utf-8"

def fs_decode(s):
    """
    >>> fs_decode('sdkjhsdfjkiï')
    u'sdkjhsdfjki\xef'
    """
    if type(s) == unicode:
        return s
    else:
        return decode(s, FSCODING)

def fs_encode(s):
    """
    >>> fs_encode(u'sdkjhsdfjkiï')
    'sdkjhsdfjki\xc3\xaf'
    >>> fs_encode('sdkjhsdfjkiï')
    'sdkjhsdfjki\xc3\xaf'
    """
    if type(s) == str:
        return s
    else:
        return encode(s, FSCODING)

def decode(s, charset='utf-8'):
    try:
        return s.decode(charset)
    except UnicodeError:
        return s.decode(charset, 'replace')

def encode(s, charset='utf-8'):
    try:
        return s.encode(charset)
    except UnicodeError:
        return s.encode(charset, 'replace')
    
